/*
*	ftrim trims file giving starting and ending point.
*
*	you might be tempted to use "dd":
*	dd -if $target -ibs 1 -iseek $size -quiet 1
*	but dd is very slow in network environment
*
*	coded by
*	Kenar (Kenji Arisawa)
*	E-mail: arisawa@aichi-u.ac.jp
*/

#include <u.h>
#include <libc.h>
#include <ctype.h>

#define BUFSIZE 16*1024


enum
{
	KB = 1024,
	MB = 1024*KB,
	GB = 1024*MB,
};


void
getunit(char *buf,int size, char *str)
{
	char *s;
	s = str;
	while(isdigit(*s++));
	snprint(buf,size,"%s",s);
}

char *usage = "usage: ftrim [-s start] [-e end] file\n";

void main(int argc, char *argv[])
{	char *a;
	vlong sp = 0;
	vlong ep = 0;
	vlong cp = 0;
	char buf[BUFSIZE];
	int n, fd;
	Dir *dir;

	ARGBEGIN{
	case 's':	a=ARGF();
		if(a == nil) sysfatal(usage);
		sp = atoll(a);
		getunit(buf,3,a);
		if(strcmp(buf,"KB") == 0)
			sp *= KB;
		else if(strcmp(buf,"MB") == 0)
			sp *= MB;
		else if(strcmp(buf,"GB") == 0)
			sp *= GB;
		break;
	case 'e':	a=ARGF();
		if(a == nil) sysfatal(usage);
		ep = atoll(a);
		getunit(buf,3,a);
		if(strcmp(buf,"KB") == 0)
			ep *= KB;
		else if(strcmp(buf,"MB") == 0)
			ep *= MB;
		else if(strcmp(buf,"GB") == 0)
			ep *= GB;
		break;
	default: sysfatal(usage);
	}ARGEND
	if(*argv == nil)
		sysfatal(usage);
	fd = open(*argv, OREAD);
	if(fd<0)
		sysfatal(usage);
	if(ep == 0){
		dir = dirfstat(fd);
		ep = dir->length;
	}
	if(seek(fd, sp, 0) < 0)
		sysfatal("fseek: %r");
	cp = sp;
	while(BUFSIZE < ep -cp && (n = read(fd, buf, BUFSIZE)) > 0){
		write(1, buf, n);
		cp += n;
	}
	n = read(fd, buf, ep - cp);
	write(1, buf, n);
	close(fd);
	/*	without exits(nil) fseek returns error status
	*	man seek(2): "Sets errstr." */
	exits(nil);
}
