#include <u.h>
#include <libc.h>
#include <bio.h>
#include "misc.h"

Rdbuf *
RdbufNew(int fd, int size)
{
	Rdbuf *p;
	p = calloc(1, sizeof(Rdbuf));
	p->fd = fd;
	p->size = size;
	p->bp = malloc(size);
	p->sp = p->cp = p->ep = nil;
	return p;
};

void
RdbufFree(Rdbuf *rb)
{
	free(rb->bp);
	free(rb);
}

int
rdlnlen(Rdbuf *rb)
{
	return rb->cp - rb->sp -1;
}

/*	Bio function Brdstr() is slow
*	Brdline() is double times better, but has a problem with EOF
*	rdline() below is similar to Brdline() but much faster
*	(as much as double times)
*	and does not have a problem with EOF	*/
char *
rdline(Rdbuf *rb)
{
	int fd, size;
	long n;
	ulong m;
	char *bp, *sp, *cp, *ep, *ep0;
	fd = rb->fd;
	size = rb->size;
	bp = rb->bp;
	sp = rb->sp;
	cp = rb->cp;
	ep = rb->ep;
	ep0 = bp + size;
	if(!sp){
		n = read(fd, bp, size);
		if(n < 0)
			sysfatal("read error %r");
		cp = bp;
		ep = bp + n;
	}

	sp = cp;
L1:
	while(cp < ep && *cp != '\n')
		cp++;
	if(*cp != '\n' || cp == ep0){
		m = cp-sp;
		if(m == size)
			sysfatal("buffer bull");
		memcpy(bp, sp, m);
		n = read(fd, bp + m, size - m);
		if(n == 0) {/* eof */
			if(m){
				sp = bp;
				cp = bp + m;
				ep = bp + (m+n);
				*cp++ = 0;
				rb->sp = sp;
				rb->cp = cp;
				rb->ep = ep;
				return sp;
			}
			return nil;
		}
		if(n < 0)
			sysfatal("read error %r");

		sp = bp;
		cp = bp + m;
		ep = bp + (m + n);
		goto L1;
	}
	/*	here, *p is NL */
	*cp++ = 0;
	rb->sp = sp;
	rb->cp = cp;
	rb->ep = ep;
	return sp;
}


/*	Why getfields() is slower than tokenize() ?
*	this is because getfields() allows utf coded chars
*	in the separater. it would be much simpler and
*	faster one if the separater is a single char and ascii	*/

int
split(char *str, char **args, int max, int sep)
{
	char *p,*s;
	int n;
	n = 0;
	for(s = p = str; *p; p++){
		if(*p == sep){
			*p = 0;
			args[n++] = s;
			if(n == max)
				return n;
			s = p + 1;
		}
	}
	args[n++] = s; /* the rest */
	return n;
}

void
strtolower(char *s)
{
	while(*s){
		*s = tolower(*s);
		s++;
	}
}

char *
skip(char *s, char *p)
{
	while(*s){
		if(!strchr(p,*s))
			break;
		s++;
	}
	return s;
}

char *
skipto(char *s, char *p)
{
	while(*s){
		if(strchr(p,*s))
			break;
		s++;
	}
	return s;
}
