/*
*	mtest is match test
*	mtest test files if given pattern exist in files
*/

#include <u.h>
#include <libc.h>
#include <ctype.h>
#include <bio.h>
#include <regexp.h>
#include "misc.h"

#define DBG if(0)
#define MAX 32

int iflag;

void
usage(void)
{
	fprint(2,"usage: mtest [-i] pattern [file ...]\n"
	"	pattern: re&re&...&re\n"
	"	it is assumed that re does not contain '&'\n"
	"output is only file names\n"
	);
	exits("usage");
}

int
chrcnt(char *s, char c)
{
	int n=0;
	for(;*s;s++)
		if(*s == c)
			n++;
	return n;
}

int
addkey(char *s, int n, char *key)
{
	if(strstr(s,key))
		return 0;
	snprint(s+strlen(s),n - strlen(s), "%s\n",key);
	return chrcnt(s,'\n');
}

int
doit(int fd, Reprog *re, char *work, int n, int nk)
{
	Biobuf bin;
	Resub match[2];
	
	char *p;
	int m,k;
	char ch;
	USED(re);
	
	Binit(&bin, fd, OREAD);
	while((p = Brdstr(&bin, '\n',1))){
		/* don't foreget the next two lines */
		if(iflag)
			strtolower(p);
		match[0].sp = p;
		match[0].ep = nil;
		while((m = regexec(re,p,match,2)) > 0){
			USED(m);
			DBG print("%d %s\n",m,p);
			DBG print("%s\n%s\n", match[0].sp, match[0].ep);
			ch = *(match[0].ep);
			*(match[0].ep) = 0;
			DBG print("%s\n", match[0].sp);
			k = addkey(work,n,match[0].sp);
			if(k == nk) /* all keys are processed */
				return 1;
			*(match[0].ep) = ch;
			p = match[0].ep;
			match[0].sp = p;
			match[0].ep = nil;
		}
	}
	return 0;
}

void
main(int argc, char *argv[])
{
	int i,n,fd,r;
	Reprog *re;
	char *args[MAX];
	char *e,*s;
	char rere[256];
	char work[256];
	int nm=0;

	ARGBEGIN{
	case 'i': iflag++;
		break;
	default: usage();
	}ARGEND

	if(*argv == nil)
		usage();

	s = strdup(argv[0]);
	if(iflag)
		strtolower(s);
	n = getfields(s, args, MAX, 1, "&");

	/*
	*	let argv[0] be r0&r1&...&rn
	*	we want to have
	*	(r0)|(r1)|...|(rn)
	*	this will be rere
	*/

	s = rere;
	e = rere + sizeof(rere);
	for(i = 0; i < n; i++){
		DBG print("%d %s\n", i, args[i]);
		s = seprint(s,e,"(%s)|",args[i]);
		DBG print("%s\n", rere);
	}
	rere[strlen(rere) - 1] = 0;
	DBG print("%s\n", rere);


	re = regcomp(rere);
	if(!re)
		sysfatal("regcomp");

	argv++;
	if(*argv == nil){
		work[0] = 0;
		r = doit(0, re,work,sizeof(work),n);
		if(r){
			print("matched\n");
			exits(nil);
		}
		print("no match\n");
		exits("no match");
	}


	while(*argv){
		fd = open(*argv,OREAD);
		if(fd < 0)
			sysfatal("%s not open",*argv);
		work[0] = 0;
		r = doit(fd, re,work,sizeof(work),n);
		DBG print("OK %d %d %s\n",n,strlen(work),work);

		if(r){
			print("%s\n",*argv);
			nm++;
		}
		close(fd);
		argv++;
	};
	if(nm)
		exits(nil);
	exits("no match");
}
